from django.contrib.auth import views, login, get_user_model, update_session_auth_hash
from django.contrib.auth.mixins import LoginRequiredMixin, UserPassesTestMixin
from django.contrib.auth.views import PasswordChangeView
from django.core.paginator import Paginator
from django.http import HttpResponseRedirect
from django.views.generic import CreateView, DetailView, UpdateView
from django.shortcuts import redirect
from django.urls import reverse_lazy

from accounts.forms import LoginForm, RegisterForm, UserUpdateForm, ProfileUpdateForm, PasswordChangeForm
from accounts.models import User, Profile


class AuthSuccessUrlMixin:
    def get_success_url(self):
        return self.request.GET.get(
            'next',
            self.request.POST.get(
                'next',
                reverse_lazy('main_page')
            )
        )


class LoginView(AuthSuccessUrlMixin, views.LoginView):
    template_name = 'login.html'
    form_class = LoginForm


class LogoutView(views.LogoutView):
    def get_next_page(self):
        return self.request.META.get('HTTP_REFERER')


class RegisterView(AuthSuccessUrlMixin, CreateView):
    model = User
    template_name = 'registration.html'
    form_class = RegisterForm

    def form_valid(self, form):
        user = form.save()
        Profile.objects.create(user=user)
        login(self.request, user)
        return redirect(self.get_success_url())


class UserDetailView(LoginRequiredMixin, DetailView):
    model = get_user_model()    # accounts.User
    template_name = 'user/detail.html'
    context_object_name = 'user'
    pk_url_kwarg = 'id'
    paginate_related_by = 5
    paginate_related_orphans = 0

    def get_context_data(self, **kwargs):
        articles = self.object.articles.order_by('-created_at')
        paginator = Paginator(
            object_list=articles,
            per_page=self.paginate_related_by,
            orphans=self.paginate_related_orphans
        )
        page_number = self.request.GET.get('page', 1)
        page = paginator.get_page(page_number)

        kwargs |= {
            'page_obj': page,
            'articles': page.object_list,
            'is_paginated': page.has_other_pages(),
            'paginator': paginator
        }
        return super().get_context_data(**kwargs)


class UserUpdateView(UserPassesTestMixin, UpdateView):
    model = get_user_model()
    form_class = UserUpdateForm
    template_name = 'user/update.html'
    pk_url_kwarg = 'id'
    context_object_name = 'user_obj'

    def test_func(self):
        return self.request.user == self.get_object()

    def get_context_data(self, **kwargs):
        if 'profile_form' not in kwargs:
            kwargs['profile_form'] = self.get_profile_form()
        return super().get_context_data(**kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        form = self.get_form()
        profile_form = self.get_profile_form()
        if form.is_valid() and profile_form.is_valid():
            profile_form.save()
            return self.form_valid(form)
        else:
            context = self.get_context_data(form=form, profile_form=profile_form)
            return self.render_to_response(context)

    def get_success_url(self):
        return reverse_lazy('user_detail', kwargs={'id': self.object.id})

    def get_profile_form(self):
        form_kwargs = {'instance': self.object.profile}
        if self.request.method == 'POST':
            form_kwargs['data'] = self.request.POST
            form_kwargs['files'] = self.request.FILES

        return ProfileUpdateForm(**form_kwargs)


class UserPasswordChangeView(UserPassesTestMixin, UpdateView):
    model = get_user_model()
    template_name = 'user/change_password.html'
    form_class = PasswordChangeForm
    pk_url_kwarg = 'id'
    context_object_name = 'user_obj'

    def test_func(self):
        return self.request.user == self.get_object()

    def form_valid(self, form):
        user = form.save()
        update_session_auth_hash(self.request, user)
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse_lazy('user_detail', kwargs={'id': self.object.id})
