from urllib.parse import urlencode

from django.contrib import messages
from django.urls import reverse, reverse_lazy
from django.db.models import Q
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin, UserPassesTestMixin
from django.core.exceptions import PermissionDenied
from django.views.generic import (
    ListView,
    DetailView,
    CreateView,
    UpdateView,
    DeleteView
)
from web.forms import ArticleModelForm, SearchForm, CommentModelForm
from web.models import Article


class ArticleIndexView(ListView):
    template_name = 'article/index.html'
    context_object_name = 'articles'
    model = Article
    ordering = ['-created_at']
    paginate_by = 3

    def dispatch(self, request, *args, **kwargs):
        self.form = self.get_search_form()
        self.search_value = self.get_search_value()
        messages.add_message(self.request, messages.SUCCESS, 'Все круто!')
        messages.add_message(self.request, messages.DEBUG, 'Debug')
        messages.add_message(self.request, messages.INFO, 'Info')
        messages.add_message(self.request, messages.WARNING, 'warning')
        messages.add_message(self.request, messages.ERROR, 'error')
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, *, object_list=None, **kwargs):
        context = super().get_context_data(**kwargs)
        context['form'] = self.form

        if self.search_value:
            context['query'] = urlencode({'search': self.search_value})

        return context

    def get_queryset(self):
        qs = super().get_queryset()
        if self.search_value:
            query = Q(title__icontains=self.search_value) | Q(author__icontains=self.search_value)
            qs = qs.filter(query)

        return qs

    def get_search_form(self):
        return SearchForm(self.request.GET)

    def get_search_value(self):
        if self.form.is_valid():
            return self.form.cleaned_data.get('search')


class ArticleCreateView(CreateView):
    template_name = 'article/article_create.html'
    model = Article
    form_class = ArticleModelForm
    permission_required = 'web.add_article'
    permission_denied_message = 'You have no rights'

    def get_success_url(self):
        return reverse('article_detail', kwargs={'id': self.object.id})


class ArticleDetailView(LoginRequiredMixin, DetailView):
    raise_exception = True
    template_name = 'article/article_detail.html'
    model = Article
    context_object_name = 'article'
    pk_url_kwarg = 'id'
    extra_context = {'form': CommentModelForm}

    def dispatch(self, request, *args, **kwargs):
        request.session['article_id'] = kwargs.get('id')
        request.session['author_id'] = request.user.id
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        return super().get_context_data(
            comments=self.object.comments.order_by('-created_at'),
            **kwargs
        )


class ArticleUpdateView(UserPassesTestMixin, UpdateView):
    model = Article
    template_name = 'article/update.html'
    form_class = ArticleModelForm
    context_object_name = 'article'
    pk_url_kwarg = 'id'
    permission_required = 'web.change_article'

    def test_func(self):
        return self.get_object().author == self.request.user or \
            self.request.user.has_perm('web.change_article')

    def get_success_url(self):
        return reverse('article_detail', kwargs={'id': self.object.id})


class ArticleDeleteView(DeleteView):
    model = Article
    template_name = 'article/delete.html'
    context_object_name = 'article'
    success_url = reverse_lazy('main_page')
    pk_url_kwarg = 'id'
